// (C) Copyright 2007 
// Douglas Gregor <doug.gregor -at- gmail.com>
// Andreas Kloeckner <inform -at- tiker.net>

// Use, modification and distribution is subject to the Boost Software
// License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

//  Authors: Douglas Gregor, Andreas Kloeckner

/** @file buffer_iface.cpp
 *
 *  This file allows sending of Python objects that expose the buffer
 *  interface.
 */


#include <boostmpi/environment.hpp>




namespace mpi = boost::mpi::python;
namespace py = boost::python;




namespace
{
  mpi::py_request isend_buffer(
      mpi::py_communicator &comm,
      int dest, int tag,
      const py::object obj)
  {
    const void *buf;
    Py_ssize_t len;
    PyObject_AsReadBuffer(obj.ptr(), &buf, &len);
    return comm.isend(dest, tag, reinterpret_cast<const char *>(buf), len);
  }




  mpi::py_request irecv_buffer(
      mpi::py_communicator &comm,
      int dest, int tag,
      const py::object obj)
  {
    void *buf;
    Py_ssize_t len;
    PyObject_AsWriteBuffer(obj.ptr(), &buf, &len);
    return comm.irecv(dest, tag, reinterpret_cast<char *>(buf), len);
  }
}



namespace boost { namespace mpi { namespace python {

void export_buffer_iface()
{
  using py::args;

  py::def("isend_buffer", isend_buffer, 
      args("comm", "dest", "tag", "vector"),
      py::with_custodian_and_ward_postcall<0,4>());
  py::def("irecv_buffer", irecv_buffer, 
      args("comm", "source", "tag", "vector"),
      py::with_custodian_and_ward_postcall<0,4>());
}

} } }
